setwd(dirname(rstudioapi::getActiveDocumentContext()$path))


set.seed(12345)
require(foreign)
dados <- read.spss("badygfinal.sav",use.value.labels=TRUE, max.value.labels=Inf, to.data.frame=TRUE)
head(dados)

p<-9  #desempenho em 9 testes 
q<-2  #fatores: 
n<-492 ## SAMPLE SIZE


#Metodo 1
ml1=factanal(~analogia+analogia+series+matrices+completa+problema+encajar+memoria+ortograf+discrimi,data=dados, factors=2)
summary(ml1)
ml1$loadings
ml1$uniquenesses
ml1
require(GPArotation)
ml1rot=factanal(~analogia+analogia+series+matrices+completa+problema+encajar+memoria+ortograf+discrimi,data=dados, factors=2,
                rotation="oblimin")
summary(ml1rot)
ml1rot$loadings
ml1rot$uniquenesses
ml1rot


#Metodo 2
require(psych)
cordados <- cor(dados[,c("analogia","completa","discrimi","encajar","matrices",
                         "memoria","ortograf","problema","series")], use="complete.obs")

ml2 <- fa(cordados,fm="ml", nfactors=2,rotate="none")
ml2
summary(ml2)
ml2$uniquenesses
ml2rot<- fa(cordados,fm="ml", nfactors=2,rotate="oblimin")
ml2rot$loadings
ml2rot$uniquenesses
ml2rot

#Metodo 3 (varias opcoes)
#Instale bfa de
#https://cran.r-project.org/src/contrib/Archive/bfa/
#install.packages("~/Downloads/bfa_0.4.tar")

library(bfa)
bayes1=bfa_gauss(~analogia+analogia+series+matrices+completa+problema+encajar+memoria+ortograf+discrimi,  data  =  dados,  num.factor  =  2,  restrict  =  NA,
                 nsim=50000, nburn  =5000,  thin  =  20, loading.prior  =  "normal")
bayes1
summary(bayes1)
bayes1$loadings
sum(bayes1$loadings)
bayes1$post.loadings.mean
bayes1$post.loadings.var
bayes1$rho
#bayes1$post.loadings
#bayes1$post.sigma2
bayes1$error_var_i
bayes1$tauinv
#bayes1$loadings.restrict
#bayes1$loadings.prob
#bayes1$post.scores
bayes1$priors
help(bfa_gauss)


#non centered
bayes1b=bfa_gauss(~analogia+analogia+series+matrices+completa+problema+encajar+memoria+ortograf+discrimi,  data  =  dados,  num.factor  =  2,  
                  restrict  =  NA, center.data=FALSE, nsim=50000, nburn  =5000,  thin  =  20, loading.prior  =  "normal")
bayes1b$loadings
bayes1b$post.loadings.mean
bayes1b$post.loadings.var
bayes1b$rho
bayes1b$tauinv

#non scaled
bayes1c=bfa_gauss(~analogia+analogia+series+matrices+completa+problema+encajar+memoria+ortograf+discrimi,  data  =  dados,  num.factor  =  2,  
                  restrict  =  NA, scale.data=FALSE, nsim=50000, nburn  =5000,  thin  =  20, loading.prior  =  "normal")
bayes1c$loadings
bayes1c$post.loadings.mean
bayes1c$post.loadings.var
bayes1c$rho
bayes1c$tauinv

#non scaled and non centered

bayes1d=bfa_gauss(~analogia+analogia+series+matrices+completa+problema+encajar+memoria+ortograf+discrimi,  data  =  dados,  num.factor  =  2,  
                  restrict  =  NA, center.data=FALSE, scale.data=FALSE, nsim=50000, nburn  =5000,  thin  =  20, loading.prior  =  "normal")
bayes1d$loadings
bayes1d$post.loadings.mean
bayes1d$post.loadings.var
bayes1d$rho
bayes1d$tauinv

#Metodo 4
require(MCMCpack)
bayes2=MCMCfactanal(~analogia+analogia+series+matrices+completa+problema+encajar+memoria+ortograf+discrimi, data=dados, factors=2,
                    burnin=5000, mcmc=50000, thin=20)
#   plot(posterior)
summary(bayes2)
help(MCMCfactanal)

#Metodo5
library(BayesFM)
head(dados)
Y=dados[,1:9]
head(Y)

mcmc <- befa(Y, Kmax =2, iter = 1000)
# post process MCMC draws to restore identification
mcmc <- post.column.switch(mcmc)
mcmc <- post.sign.switch(mcmc)
summary(mcmc)  # summarize posterior results
plot(mcmc)     # plot posterior results
# summarize highest posterior probability (HPP) model
summary(mcmc, what = 'hppm')


#Metodo 6 Bayesian estimation using WinBUGS
names(dados)

attach(dados)

y=matrix(c(analogia,series,matrices,completa,problema,encajar,memoria,ortograf,discrimi),492,9)
head(y)
library("R.utils")
#y<-std.data(y,c(y[,1],y[,2],y[,3],y[,4],y[,5],y[,6],y[,7,y[,8],y[,9])) #escala padronizada

library(BRugs)
library(R2WinBUGS)
#library(smoothSurv)
#library(mvtnorm)
#library(mnormt)

library("R.utils")

####################################

### ProgramaBUGS para Analise Fatorial model

bfa.model <- function() {
  # Likelihood
  for (i in 1:492){ 
    for(j in 1:9){          
      y[i,j]~dnorm(media[i,j],Psi1[j])
      media[i,j]<-Lambda[1,j]*z[i,1]+Lambda[2,j]*z[i,2]
    }
  }
  
  
  #latent effect
  for (i in 1:492){ 
    z[i,1:2]~dmnorm(muL[1:2],Omega1[,])
  }
  
  # Priors
  for (j in 1:9){
    Lambda[1,j]~dnorm(mubG1[j],tau)
    Lambda[2,j]~dnorm(mubG2[j],tau)
    
    Psi1[j]~dgamma(2,0.1)
    Psi[j]<-1/Psi1[j]
  }
  
  tau.inv~dgamma(0.01,0.01)
  tau<-1/tau.inv
  
  Omega1[1:2,1:2]<-inverse(Omega[,])  
}


####################################################

#MCMCfactanal usa PSI IG(0.001/2,0.001/2) Y LAMBDA N(0,1000)
#EN FORMATO WINBUGS LAMBDA N(0,0.001)    Psi1[j]~dgamma(0.001,0.001) y Psi[j]<-1/Psi1[j]}
#no recomendable, o en todo caso, usar uniforme o inversa gamma con el openbugs que tiene esa distribucion

write.model(bfa.model,"bfa.bug")

#### Ajustando com WinBUGS
datasN<-list(y=dput(y))
Omega<-diag(q) #AFE
Omega<-0.7*diag(q) + 0.3*matrix(1,q,q)
muL<- rep(0,q)
mubG1<- -0.4*rep(1,p)
mubG2<- 0.4*rep(1,p)  
datasN <- list("y","muL","Omega","mubG1","mubG2")
modelN<- file.path(pasta, "bfa.bug")

parametersN<- c("Lambda","Psi")
niter<-20000
initsN <- function(){list(tau.inv=100,Psi1=rep(0.4,p),Lambda=rep(0.5,p))}


print(paste("FAT - Initial Time: ",date(),sep=""))

bayes3 <- bugs(datasN, initsN, parametersN, modelN,
               n.chains=1, n.iter=niter, n.thin=1 , debug=TRUE, codaPkg=FALSE,
               n.burnin=1000,program="winbugs", working.directory=pasta,digits = 5,clearWD = TRUE)

print(resu.simN,5)
#file.remove("bfa.bug")



#si se coloca openbugs o openBUGS se require instalar Brugs#. Caso contrario, tener
#instalado el winbugs y direccionar el archivo de trabajo#

#Metodo 7 Usando Stan
#https://rfarouni.github.io/2015-04-26-fa/

